//---------------------------------------------------------------------------

#ifndef acefileH
#define acefileH

//---------------------------------------------------------------------------

const int MAXZLIBBUFSIZE = 16384*1024;

struct KujuFileHead {
        DWORD Sign1;
        DWORD Sign2;
        int dataSize;
        DWORD Sign3;

        bool isKujuHead() { return Sign1==0x494d4953 && Sign3==0x40404040 &&
                                (Sign2==0x46404153 || Sign2==0x40404153); }
        bool isZCompressed() { return Sign1==0x494d4953 && Sign3==0x40404040 &&
                                Sign2==0x46404153; }
        bool notZCompressed() { return Sign1==0x494d4953 && Sign3==0x40404040 &&
                                Sign2==0x40404153; }
        int getZBufSize() { return (isZCompressed()&&dataSize<MAXZLIBBUFSIZE)?dataSize:-1; };

        void prepareCompressedHead(int size){ Sign1=0x494d4953; Sign2=0x46404153;
                     dataSize=size; Sign3=0x40404040; }
        void prepareUnCompressedHead() { Sign1=0x494d4953; Sign2=0x40404153;
                     dataSize=0x40404040; Sign3=0x40404040; }
};
//---------------------------------------------------------------------------

struct AceHead {                        // total 248+16/32 bytes (trans/alpha)
        DWORD fileType;                 // 1 for ACE texture
        DWORD textureType;              // 0=no_mip 1 4 5=mip 11 15=dxt  10=dxt_no_mip
        DWORD width;                             // 1 4 10 11 by Photoshop plugin
        DWORD height;
        DWORD colorType;                // 0x0e=Opaque 0x10=Trans 0x11=alpha 0x12=DXT
        DWORD colorDepth;               // 3=Opaque 4=Trans 5=alpha
        DWORD reserved;                 // always 0
        char UnknownString[16];         // "Unknown" string
        char MemoString[80];            // texture maker infomation
        DWORD unknown1;                 // ox0e / 0x10 (opaque/alpha) when DXT, otherwise 0
        DWORD unknown2;                 // 0x12  when DXT, otherwise 0
        DWORD FixedData[17];            // 0 0 0_ 8 3 8 4 8 5
};
//---------------------------------------------------------------------------

enum ACEColorDepths { Opaque,Trans,Alpha,Unknown };     // Unknown for file can not be recognized
enum ACECompressType { NoCompress,ZLib,DXT1,DXT1ZLib,
                       ZLib2D,ReMip };          // Last 2 Type only for select compress method
                                                // and will not appear in AceInfo struct

struct AceInfo {
        int width;
        int height;
        bool isZLib;
        bool isDXT1;
        bool hasMIPMAP;
        AnsiString MemoStr;
        ACEColorDepths colorDepth;
        ACECompressType compressType;
        AnsiString infoMsg;

        bool allow3D() { return width==height && ( width==32||width==64||width==128
                                        ||width==256||width==512||width==1024 ); }
        bool allowDXTEx() { return colorDepth<Alpha && allow3D(); }
        bool allowDXT() { return colorDepth<Alpha && allow3D() && width<=512; }
        int EstDXTFileSize();

        bool byAceIt () { return MemoStr.Pos("AceIt"); }
        bool byReMipIt() { return MemoStr.Pos("ReMipIt"); }
        // Maker= AceIt ReMipIt mwdds.dll MakeACE Photoshop plug-in
};

//---------------------------------------------------------------------------

class AceFile : public AceInfo {
        KujuFileHead *kujuHead;
        AceHead *aceHead;
        unsigned char *dataBuffer;              // delete when deconstuct, init NULL
        unsigned char *imageData;
        int headerSize;
        int loadSize;

        ACECompressType compMethod;

        void parseHeader();
        void makeHeader();
public:
        AnsiString fileName;
        int fileSize;
        AnsiString errorMsg;

        AceFile() : dataBuffer(NULL) { colorDepth=Unknown; };
        AceFile(const AceFile & AceFile,ACECompressType CompMethod);
        ~AceFile() { if(dataBuffer) delete dataBuffer; };

        AceInfo GetInfo(AnsiString FileName);
        void DecideCompressMethod();

        bool LoadFromFile(AnsiString FileName=AnsiString());
        bool SaveToFile();

        bool SaveToTGA();
        bool LoadFromTGA();
};

//---------------------------------------------------------------------------
// ZLib compress/decompress Kuju file use zlibwapi.dll

AnsiString CompDecompKujuFile(const AnsiString srcname,const AnsiString destname,
                        bool compress=true,float ratio=1.0,int compresslevel=6);
//---------------------------------------------------------------------------

#endif
