//---------------------------------------------------------------------------
// This software is released under GNP General Public License (GPL)

#include <vcl.h>
#pragma hdrstop
#include <mmsystem.h>

#include "main.h"
#include "mylib.h"
#include "acefile.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)
#pragma link "cdiroutl"
#pragma resource "*.dfm"
TMainForm *MainForm;

//---------------------------------------------------------------------------

/* define in aceopt.cpp
extern AnsiString makeacepath;
extern AnsiString extraexepath;   */

enum TextureType { is2D=0, is3D=1, typeUnknown=-1 };

// variable for stop long time process
bool Processing=false,StopProcess;

// Variable for Count
int ScanDirs,ScanFiles,CompressFiles,CompressDXT,Errors;
long DiskSpaceSave;

const char *FileTypeName[5] = {
        "Not compressed files   : ",
        "ZLib compresed files   : ",
        "DXT compresed files   : ",
        "DXT+ZLib compresed : ",
        "Unrecognized files       : "
        };
int BefroeFileTypeCount[5];
long BefroeFileSizeSum[5];
int AfterFileTypeCount[5];
long AfterFileSizeSum[5];

// String array for determine compress method based on dir and file name
TStringList * AllowPDir;
TStringList * AllowPPDir;
TStringList * PathMustContain;
TStringList * DenyPDir;
TStringList * DenyPPDir;
TStringList * DenyPathContainPhase;
TStringList * DenyFileInSameDir;
TStringList * DenyTextureName;
TStringList * NoDXTFileName;
TStringList * DirPath;

//---------------------------------------------------------------------------

__fastcall TMainForm::TMainForm(TComponent* Owner)
        : TForm(Owner)
{
        // Alloc memory for strings
        AllowPDir = new TStringList;
        AllowPPDir = new TStringList;
        PathMustContain = new TStringList;
        DenyPDir = new TStringList;
        DenyPPDir = new TStringList;
        DenyPathContainPhase = new TStringList;
        DenyFileInSameDir = new TStringList;
        DenyTextureName = new TStringList;
        NoDXTFileName = new TStringList;
        DirPath = new TStringList;

        // MSTS install path
        DirEdit->Text =  GetRegistryValue(
                "Software\\Microsoft\\Microsoft Games\\Train Simulator\\1.0",
                "Path");

        // Load help.txt
        AnsiString helppath =
                IncludeTrailingPathDelimiter( GetCurrentDir() )+"quickstart.txt";
        if( !FileExists(helppath) )
                helppath =IncludeTrailingPathDelimiter( GetCurrentDir() )+"help.txt";
        if( FileExists(helppath) )
                LogMemo->Lines->LoadFromFile(helppath);


        // Load mwace.dll & mwdds.dll
        if( !LoadMwace() ) {
                Application->MessageBox(
                        "Can not load mwace.dll or version old!\n"
                        "You must have mwace.dll installed to run this program!\n"
                        "Program quit!",
                        "Can not load mwace.dll!",MB_OK+MB_ICONERROR);
                Application->Terminate();
                return;
         }

        if( !LoadMwdds() )
                UseMwddsCheckBox->Enabled = false;
        else {
                if( GetMWDDSVersion()<51 ) {
                        Application->MessageBox(
                        "Acebcomp need 4.0.0.51 version or above to work!\n"
                        "Please download newest version!",
                        "Your mwdds.dll version is old!",MB_OK+MB_ICONWARNING);
                        UseMwddsCheckBox->Enabled = false;
                 }
         }

        // Check does AceIt.exe or ReMipIt.exe exists
        if( FileExists(ExtProgPathEdit->Text) ) {
                UseAceItForAllCheckBox->Checked = true;
                ReMipIt1024CheckBox->Checked = true;
         }
        else if( FileExists("C:\\Program Files\\ReMipIt\\remipit.exe") )
                ReMipIt1024CheckBox->Checked = true;

        // check makeace.exe path
        makeacepath = IncludeTrailingPathDelimiter( GetCurrentDir() )+"makeace.exe";
        if( !FileExists( makeacepath ) )
                makeacepath = DirEdit->Text + "\\UTILS\\makeace.exe";
        if( !FileExists( makeacepath ) ) {
                Application->MessageBox(
                        "Generally you need MakeACE.exe in MSTS Utils directory!\n"
                        "to compress ACE texture files. Unless you have\n"
                        "both mwdds.dll and ReMipIt.exe or AceIt.exe installed!\n",
                        "Can not find MakeACE.exe!",MB_OK+MB_ICONWARNING);
         }
        else if( getfilesize(makeacepath)!=184320 ) {
                Application->MessageBox(
                        "Your makeace.exe file size is not same as the newest version!\n"
                        "That mean you do not have MSTS 1.1 update patched or it's\n"
                        "infected by virus! Older version lack some function!",
                        "MakeACE.exe size is wrong!",MB_OK+MB_ICONWARNING);
         }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FormClose(TObject *Sender, TCloseAction &Action)
{
        delete AllowPDir;
        delete AllowPPDir;
        delete PathMustContain;
        delete DenyPDir;
        delete DenyPPDir;
        delete DenyPathContainPhase;
        delete DenyFileInSameDir;
        delete DenyTextureName;
        delete NoDXTFileName;
        delete DirPath;

        UnloadMwdds();
        UnloadMwace();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::SelectDirButtonClick(TObject *Sender)
{
        AnsiString root,directory;

        directory = DirEdit->Text;
        if( SelectDirectory("Select a directory to compress",root,directory) )
                DirEdit->Text = directory;
}
//---------------------------------------------------------------------------
// Below for Log Tab

void __fastcall TMainForm::CheckOnlyCheckBoxClick(TObject *Sender)
{
        if( CheckOnlyCheckBox->Checked )
                CompressButton->Caption = "Check Files";
        else
                CompressButton->Caption = "Compress";
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::SkipNormCompMsgCheckBoxClick(TObject *Sender)
{
        SkipNoOpMsgCheckBox->Enabled = !SkipNormCompMsgCheckBox->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::CopyToClipboardButtonClick(TObject *Sender)
{
        LogMemo->SelectAll();
        LogMemo->CopyToClipboard();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::HelpButtonClick(TObject *Sender)
{
        AnsiString helppath =
                IncludeTrailingPathDelimiter( GetCurrentDir() )+"help.txt";
        if( !FileExists(helppath) )
                helppath =IncludeTrailingPathDelimiter( GetCurrentDir() )+"quickstart.txt";
        if( FileExists(helppath) ) {
                LogMemo->Font->Size = 10;
                LogMemo->Lines->LoadFromFile(helppath);
         }
        else {
                Application->MessageBox(
                        "Help.txt not found!\n",
                        "Help not found!",MB_OK+MB_ICONERROR);
         }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::StatusBarResize(TObject *Sender)
{
      StatusBar->Panels->Items[0]->Width = StatusBar->Width -236;    // 636-400
}
// Above for Log Tab
//---------------------------------------------------------------------------
// Below for Compress & Texture type Option Tab

void __fastcall TMainForm::Check3DCheckBoxClick(TObject *Sender)
{
        AllowPDirLabel->Enabled = Check3DCheckBox->Checked;
        AllowPPDirLabel->Enabled = Check3DCheckBox->Checked;
        PathMustContainLabel->Enabled = Check3DCheckBox->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::NoDXTCheckBoxClick(TObject *Sender)
{
        ForceDXTCheckBox->Enabled = !NoDXTCheckBox->Checked;
        NoDXTFileNameLabel->Enabled = !NoDXTCheckBox->Checked;
        ForceTransCheckBox->Enabled = !NoDXTCheckBox->Checked;
        if( NoDXTCheckBox->Checked ) ForceTransCheckBox->Checked = false;

        if( NoDXTCheckBox->Checked )
                Application->MessageBox(
                        "Please do not check No DXT compress option!\n"
                        "\n"
                        "In theory DXT1 is a lossy compress format will reduce graphics\n"
                        "quality. But in screenshot of game, it's very difficult to\n"
                        "distinguish the difference. So generally you don't need turn off\n"
                        "DXT1 compress option. Why I write this program is also primary\n"
                        "for recompress these ZLib compressed textures to DXT1 format.",
                        "Don't use it!",MB_OK+MB_ICONINFORMATION);
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ForceTransCheckBoxClick(TObject *Sender)
{
        if( ForceTransCheckBox->Checked )
                Application->MessageBox(
                        "Please don't check Force Trans option as normal operation!\n"
                        "\n"
                        "Force texture with 8 bit alpha channel to 1 bit alpha channel\n"
                        "no doubt will degrade graphics quality! This option is designed\n"
                        "for allow alpha texture be DXT compressed and will not change\n"
                        "ZLib compress result. It's a experimental option, so you should\n"
                        "use it carefully, move files you want to try to a temporary\n"
                        "directory for process, and always backup!\n"
                        " (better manually backup first than using backup option)",
                        "Don't use it!",MB_OK+MB_ICONWARNING);

                // Force trans will Degrade graphics quality, for special process only,
                // only for allow DXT format for many files, ZLib alpha will not change
                // User should carefully only process these files they have checked
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::AddMipTo3DCheckBoxClick(TObject *Sender)
{
        AddMip1024OnlyCheckBox->Enabled = !AddMipTo3DCheckBox->Checked;
}
// Above for Compress & Texture type Option Tab
//---------------------------------------------------------------------------
// Below for Misc Option Tab

void __fastcall TMainForm::VerifyZLibCompCheckBoxClick(TObject *Sender)
{
        if( VerifyZLibCompCheckBox->Checked )
                NoBackupZLibCheckBox->Checked = true;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::KeepTGAFileCheckBoxClick(TObject *Sender)
{
        DXTOnlyCheckBox->Enabled = KeepTGAFileCheckBox->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::UseMwddsCheckBoxClick(TObject *Sender)
{
        DitherCheckBox->Enabled = UseMwddsCheckBox->Checked;
        MipFilterLabel->Enabled = UseMwddsCheckBox->Checked;
        MipFilterComboBox->Enabled = UseMwddsCheckBox->Checked;
}
//---------------------------------------------------------------------------

// Event handel shared by 3 RadioButton  for detect uncheck
void __fastcall TMainForm::NoBackupRadioButtonClick(TObject *Sender)
{
        NoBackupZLibCheckBox->Enabled = !NoBackupRadioButton->Checked;

        PutCompToBckDirCheckBox->Enabled = MoveFileBackupRadioButton->Checked;
}
//---------------------------------------------------------------------------

// Event handel shared by 5 CheckBoxs
void __fastcall TMainForm::UseAceItForAllCheckBoxClick(TObject *Sender)
{
        if( Sender==AceIt1024ZLibCheckBox && AceIt1024ZLibCheckBox->Checked )
                ReMipIt1024CheckBox->Checked = true;

        bool aceit,remipit;

        aceit = UseAceItForAllCheckBox->Checked
                || AceIt1024DXTCheckBox->Checked
                || AceIt1024ZLibCheckBox->Checked;
        remipit = !aceit && ( ReMipItCheckBox->Checked
                           || ReMipIt1024CheckBox->Checked );

        if( Sender==UseAceItForAllCheckBox ) {
                AceIt1024DXTCheckBox->Enabled = !UseAceItForAllCheckBox->Checked;
                AceIt1024ZLibCheckBox->Enabled = !UseAceItForAllCheckBox->Checked;
         }
        ReMipIt1024CheckBox->Enabled = !ReMipItCheckBox->Checked;

        if( !remipit ) {
                if( SelectExePathLabel->Caption=="Select ReMipIt.exe path: " ) {
                        SelectExePathLabel->Caption = "Select AceIt.exe path: ";
                        ExtProgPathEdit->Text = "C:\\Program Files\\AceIt\\aceit.exe";
                        OpenDialog->FileName = "aceit.exe";
                 }
         }
        else {
                if( SelectExePathLabel->Caption=="Select AceIt.exe path: " ) {
                        SelectExePathLabel->Caption = "Select ReMipIt.exe path: ";
                        ExtProgPathEdit->Text = "C:\\Program Files\\ReMipIt\\remipit.exe";
                        OpenDialog->FileName = "remipit.exe";
                 }
         }
}
//---------------------------------------------------------------------------

// also shared by BackupDirEdit->OnExit event
void __fastcall TMainForm::SelectBackupDirButtonClick(TObject *Sender)
{
        AnsiString root,directory;
        bool selected;

        directory = IncludeTrailingPathDelimiter(BackupDirEdit->Text);

        if( Sender==SelectBackupDirButton ) {
                selected =
                        SelectDirectory("Select the temporary directory for backup",
                                        root,directory);
                if( selected )
                        BackupDirEdit->Text = directory;
         }

        // check not allow under main path
        if( IncludeTrailingPathDelimiter(DirEdit->Text).AnsiCompareIC(
                directory.SubString(1,DirEdit->Text.Length()+1) )==0 ) {
                        Application->MessageBox(
                        "Backup directory can not locate under compress directory!\n"
                        "Please select again!",
                        "Wrong directory location!",MB_OK+MB_ICONERROR);
         }
        else {
                if( selected )
                        MoveFileBackupRadioButton->Checked = true;
         }
        // need check again before run
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::SelectExePathButtonClick(TObject *Sender)
{
        OpenDialog->FileName = ExtProgPathEdit->Text;
        if( OpenDialog->Execute() )
                ExtProgPathEdit->Text = OpenDialog->FileName;

}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ExtEditExit(TObject *Sender)
{
        if( ExtEdit->Text.IsEmpty() ||               // not allow to .ace again for avoid dead loop
                        ExtractFileExt(ExtEdit->Text).AnsiCompareIC(".ace")==0 )
                ExtEdit->Text = ".bak";

}
//---------------------------------------------------------------------------
// Above for Misc Option Tab

//---------------------------------------------------------------------------

void __fastcall TMainForm::PageControlChange(TObject *Sender)
{
        if( Processing )                        // No change while process
                PageControl->ActivePage = LogTabSheet;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::StopButtonClick(TObject *Sender)
{
        if( Application->MessageBox(
                "Are you want to abort process?","Confirm Abort",MB_YESNO )==IDYES )
                StopProcess = true;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::CompressButtonClick(TObject *Sender)
{
        // compress directory exist
        DirEdit->Text = ExcludeTrailingPathDelimiter(DirEdit->Text);
        AnsiString BaseDir = DirEdit->Text;
        if( !DirectoryExists(BaseDir) ) {
                StatusBar->Panels->Items[0]->Text = "Directory not exist!";
                return;
         }

  if( !CheckOnlyCheckBox->Checked ) {
        // check makeace path
        if( !UseAceItForAllCheckBox && !( ReMipIt1024CheckBox->Checked
                    && UseMwddsCheckBox && UseBuiltinZLibCompCheckBox->Checked )
            && !FileExists( makeacepath ) ) {
                Application->MessageBox(
                        "MakeACE.exe not exist in program directory!",
                        "MakeACE.exe not found!",MB_OK+MB_ICONERROR);
                 return;
         }
        // check extra program path
        extraexepath = ExtProgPathEdit->Text;
        if( UseAceItForAllCheckBox->Checked
                || AceIt1024DXTCheckBox->Checked
                || AceIt1024ZLibCheckBox->Checked
                || ReMipItCheckBox->Checked
                || ReMipIt1024CheckBox->Checked ) {
                if( !FileExists( extraexepath ) ) {
                        Application->MessageBox(
                        ("Extra compress program not exist:\n"
                        +extraexepath+
                        "\nPlease select again!").c_str(),
                        "No extra program you choiced!",MB_OK+MB_ICONERROR);
                        return;
                 }
         }
        // check backup path to avoid locate under compress directory
        if( MoveFileBackupRadioButton->Checked &&
                IncludeTrailingPathDelimiter(DirEdit->Text).AnsiCompareIC(
                IncludeTrailingPathDelimiter(BackupDirEdit->Text).SubString(
                        1,DirEdit->Text.Length()+1) )==0 ) {
                        Application->MessageBox(
                        "Backup directory can not locate under compress directory!\n"
                        "Please select again!",
                        "Wrong directory location!",MB_OK+MB_ICONERROR);
                return;
         }
        // force backup path
        if( MoveFileBackupRadioButton->Checked ) {
                if( BackupDirEdit->Text.IsEmpty() ) {
                        Application->MessageBox(
                        "You have not assigned backup directory yet!\n"
                        "Please input before use backup directory method!\n"
                        "If you don't want backup, please select don't backup first!",
                        "No backup directory!",MB_OK+MB_ICONERROR);
                        return;
                 }
                if( !DirectoryExists(BackupDirEdit->Text) ) {
                        if( ForceDirectories( BackupDirEdit->Text ) )
                                Application->MessageBox(
                        ("Backup directory:\n"
                        + BackupDirEdit->Text +
                        "\nHave been created!").c_str(),
                        "Backup directory created!",MB_OK+MB_ICONINFORMATION);
                        else {
                                Application->MessageBox(
                        ("Can not create backup directory:\n"
                        + BackupDirEdit->Text +
                        "\nPlease input again!").c_str(),
                        "Can not create backup directory!",MB_OK+MB_ICONERROR);
                                return;
                         }
                 }
         }
  }
  
        // Prepare for compare stings
        SplitString(AllowPDir,AllowPDirEdit->Text);
        SplitString(AllowPPDir,AllowPPDirEdit->Text);
        SplitString(PathMustContain,PathMustContainEdit->Text);
        SplitString(DenyPDir,DenyPDirEdit->Text);
        SplitString(DenyPPDir,DenyPPDirEdit->Text);
        SplitString(DenyPathContainPhase,DenyPathContainPhaseEdit->Text);
        SplitString(DenyFileInSameDir,DenyFileInSameDirEdit->Text);
        SplitString(DenyTextureName,DenyTextureNameEdit->Text);
        SplitString(NoDXTFileName,NoDXTFileNameEdit->Text);

        // No change while process
        Processing = true;
        StopProcess = false;
        StopButton->Enabled = true;
        DirEdit->ReadOnly = true;
        SelectDirButton->Enabled = false;
        CheckOnlyCheckBox->Enabled = false;
        CompressButton->Enabled = false;
        RecurseDirCheckBox->Enabled = false;
        HelpButton->Enabled = false;

        // Clear counts
        ScanDirs=0;
        ScanFiles=0;
        CompressFiles=0;
        CompressDXT=0;
        DiskSpaceSave=0;
        Errors=0;
        for(int i=0;i<5;i++ ) BefroeFileTypeCount[i] = 0;
        for(int i=0;i<5;i++ ) BefroeFileSizeSum[i] = 0;
        for(int i=0;i<5;i++ ) AfterFileTypeCount[i] = 0;
        for(int i=0;i<5;i++ ) AfterFileSizeSum[i] = 0;

        // Ready to show
        PageControl->ActivePage = LogTabSheet;
        LogMemo->Lines->Clear();
        LogMemo->Font->Size = 8;
        StatusBar->Panels->Items[1]->Text = "0/0";
        StatusBar->Panels->Items[2]->Text = AnsiString ();

        // Recursive call
        long start = timeGetTime();
    try {
        ProcessOneDir(BaseDir);
     }
    __finally {

        // User Abort message
        if( StopProcess ) {
                LogMemo->Lines->Add("");
                LogMemo->Lines->Add("Process Aborted!");
         }

        // Enable controls
        StopButton->Enabled = false;
        DirEdit->ReadOnly = false;
        SelectDirButton->Enabled = true;
        CheckOnlyCheckBox->Enabled = true;
        CompressButton->Enabled = true;
        RecurseDirCheckBox->Enabled = true;
        HelpButton->Enabled = true;
        Processing = false;

        // Statics message
        LogMemo->Lines->Add("");                // Statics count
        LogMemo->Lines->Add("Directorys with textures :"+AnsiString(ScanDirs)
                        +"   Scan Texture Files :"+AnsiString(ScanFiles)
                        +"   Compress Files :"+AnsiString(CompressFiles)
                        +"   ( DXT method :"+AnsiString(CompressDXT)+" )"
                        +"   Errors :"+AnsiString(Errors) );
        LogMemo->Lines->Add("Total File Size Saved :  "
                +FloatToStrF(DiskSpaceSave/(1024*1024.0),ffNumber,18,2)
                +" MB     Use time : "
                +AnsiString( (timeGetTime()-start)*0.001 )+" seconds" );
        StatusBar->Panels->Items[0]->Text = AnsiString();


        // Before compress File type statics count
        AnsiString str1,str2;
        LogMemo->Lines->Add("");
        LogMemo->Lines->Add("Before compress :");
        LogMemo->Lines->Add("");
        for(int i=0;i<5;i++ ) {
                str1 = AnsiString(BefroeFileTypeCount[i]);
                if( BefroeFileTypeCount[i]>0 ) {
                        str1 += AnsiString::StringOfChar(' ',15-2*str1.Length());

                        str2 = FloatToStrF(BefroeFileSizeSum[i]/(1024*1024.0),ffNumber,18,2);
                        str2 = " Size : " + str2 + " MB"
                               + AnsiString::StringOfChar(' ',23-2*str2.Length())
                               + " Average File size : " +
                               FloatToStrF(BefroeFileSizeSum[i]/(BefroeFileTypeCount[i]*1024.0)
                                        ,ffNumber,18,1)
                               + " KB";
                 }
                else
                        str2 = AnsiString ();

                LogMemo->Lines->Add( FileTypeName[i] + str1 + str2 );
         }

        // After compress File type statics count
        if( CheckOnlyCheckBox->Checked )
                return;
        for(int i=0;i<5;i++ ) {
                AfterFileTypeCount[i] += BefroeFileTypeCount[i];
                AfterFileSizeSum[i] += BefroeFileSizeSum[i];
         }
        LogMemo->Lines->Add("");
        LogMemo->Lines->Add("After compress :");
        LogMemo->Lines->Add("");
        for(int i=0;i<5;i++ ) {
                str1 = AnsiString(AfterFileTypeCount[i]);
                if( AfterFileTypeCount[i]>0 ) {
                        str1 += AnsiString::StringOfChar(' ',15-2*str1.Length());

                        str2 = FloatToStrF(AfterFileSizeSum[i]/(1024*1024.0),ffNumber,18,2);
                        str2 = " Size : " + str2 + " MB"
                               + AnsiString::StringOfChar(' ',23-2*str2.Length())
                               + " Average File size : " +
                               FloatToStrF(AfterFileSizeSum[i]/(AfterFileTypeCount[i]*1024.0)
                                        ,ffNumber,18,1)
                               + " KB";
                 }
                else
                        str2 = AnsiString ();

                LogMemo->Lines->Add( FileTypeName[i] + str1 + str2 );
         }
    }
}
//---------------------------------------------------------------------------

void TMainForm::ProcessOneDir(AnsiString DirName)
{
        AnsiString filemsg,backupdir;
        int dirallow3d;
        long disksavebegin;
        int msglevel,filescountbegin;
        TSearchRec sr;

        // Count before process this dir
        disksavebegin = DiskSpaceSave;
        filescountbegin = CompressFiles;

        DirName = ExcludeTrailingPathDelimiter(DirName);
        // Calc releative backup directory
        if( MoveFileBackupRadioButton->Checked )
                backupdir = IncludeTrailingPathDelimiter(
                        ExcludeTrailingPathDelimiter(BackupDirEdit->Text)
                        + DirName.SubString(
                                DirEdit->Text.Length()+1,
                                DirName.Length()-DirEdit->Text.Length() ) );

        // Scan Texture Files
        int begin = DateTimeToFileDate( Now() );
        if( FindFirst(IncludeTrailingPathDelimiter(DirName)+"*.ace",
                        faArchive|faReadOnly,sr) == 0 ) {
                dirallow3d = CheckDirTextureType(DirName);
                if( dirallow3d==is2D )
                        LogMemo->Lines->Add("Scan: "+DirName+"     2D Textute only");
                else if( dirallow3d==is3D )
                        LogMemo->Lines->Add("Scan: "+DirName+"     Allow 3D Textute");
                else {
                        LogMemo->Lines->Add("Scan: "+DirName+"     Textute type unknown");
//                        ShowMessage("Dir Textute type unknown");
                 }
                ScanDirs++;

                do {
                        DoMessageLoop();           // check stop button
                        if( StopProcess ) return;

                        // for avoid process processed file again in FAT32 system
                        if( sr.Time>=begin-1 && sr.Time<=DateTimeToFileDate(Now())+1 )
                                continue;       // -1 +1 for FAT file time is store by 2 second interval

                        ScanFiles++;
                        StatusBar->Panels->Items[0]->Text = "  "+
                                MinimizeName(IncludeTrailingPathDelimiter(DirName)+sr.Name,
                                                StatusBar->Canvas,
                                                StatusBar->Panels->Items[0]->Width-24);
                        StatusBar->Update();

                        filemsg = ProcessOneFile(
                                        IncludeTrailingPathDelimiter(DirName),
                                        sr.Name,sr.Size,dirallow3d,
                                        backupdir,msglevel);

                        StatusBar->Panels->Items[1]->Text =
                                AnsiString(CompressFiles)+"/"+AnsiString(ScanFiles);

                        // Suppress low level message
                        if( msglevel==2 ||
                            ( msglevel==1 && !SkipNormCompMsgCheckBox->Checked ) ||
                            ( SkipNoOpMsgCheckBox->Enabled && !SkipNoOpMsgCheckBox->Checked ) )
                                        LogMemo->Lines->Add(filemsg);

                        if( StopProcess ) return;
                } while ( FindNext(sr) == 0 );

                FindClose(sr);
         }
        else
                LogMemo->Lines->Add("Scan: "+DirName);


        // Search sub-directories recursive
        if( RecurseDirCheckBox->Checked &&
            FindFirst(IncludeTrailingPathDelimiter(DirName)+"*.*",
                        faDirectory,sr) == 0 ) {
                do {
                        DoMessageLoop();           // check stop button
                        if( StopProcess ) return;

                        if( sr.Name=="." || sr.Name==".." )
                                continue;
                        if ( (sr.Attr & faDirectory) == faDirectory ) {
                                ProcessOneDir(
                                        IncludeTrailingPathDelimiter(DirName)+sr.Name);
                         }
                } while ( FindNext(sr) == 0 );

                FindClose(sr);
         }

        if( DiskSpaceSave>disksavebegin )
                LogMemo->Lines->Add("Total save "
                                +FloatToStrF(DiskSpaceSave-disksavebegin,ffNumber,18,0)
                                +" Bytes ("+AnsiString(CompressFiles-filescountbegin)
                                +" files) under: "+DirName );
}
//---------------------------------------------------------------------------

// Decide use which method for compress
AnsiString TMainForm::ProcessOneFile(AnsiString path,AnsiString fname,
        int filesize,int dirallow3d,AnsiString backupdir,int &msglevel)
{
        AnsiString filepath,fileinfomsg,compmsg;
        AceInfo aceinfo;
        ACECompressType compmethod = NoCompress;

        // Get ACE file info
        filepath = IncludeTrailingPathDelimiter(path)+fname;

        AceFile acefile;
        aceinfo = acefile.GetInfo(filepath);
        if( aceinfo.colorDepth==Unknown ) {
                BefroeFileTypeCount[4]++;
                BefroeFileSizeSum[4] += acefile.fileSize;
         }
        else {
                BefroeFileTypeCount[aceinfo.compressType]++;
                BefroeFileSizeSum[aceinfo.compressType] += acefile.fileSize;
         }

        // Generic file info
        fileinfomsg = " "+fname+"  ("+(filesize+1023)/1024+"kb)  "
                //                + AnsiString(aceinfo.width)+"x"+AnsiString(aceinfo.height)+"   "
                + aceinfo.infoMsg;
        msglevel = 0;           // msglevel 0: every file, 1: compress, 2: error or special message


        // Texture unknown format!
        if( aceinfo.colorDepth==Unknown ) {
                msglevel = 2;
                Errors++;
                return fileinfomsg+" - bad file - Error and Skip!";
         }


        // DXT1 & ZLib format compressed, always skip
        if( aceinfo.compressType==DXT1ZLib )
                        return fileinfomsg+"   DXT & ZLib compressed - skip";


        // DXT1 format compressed, ZLib compress
        if( aceinfo.compressType==DXT1 )
                if( !DXTZlibCheckBox->Checked )
                        return fileinfomsg+"   DXT compressed - skip";
                else
                        compmethod = DXT1ZLib;


        // 2D Texture part
        bool fnameallow3d = !MatchesMaskStrings(DenyTextureName,fname);
        bool fnameallowdxt = !MatchesMaskStrings(NoDXTFileName,fname);
        bool targetMIP = aceinfo.hasMIPMAP;

        if( !fnameallow3d ) dirallow3d = is2D;
        if( dirallow3d==is3D && ( AddMipTo3DCheckBox->Checked
            || ( AddMip1024OnlyCheckBox->Checked && aceinfo.width==1024 ) ) )
                targetMIP = true;
        if( dirallow3d==is2D && NoMipFor2DCheckBox->Checked )
                targetMIP = false;



        if( compmethod==NoCompress &&
            ( !targetMIP || !aceinfo.allow3D() ) ) {
                if( aceinfo.hasMIPMAP )
//                        ShowMessage("2D Texture has MIP map!");
                        msglevel = 2;

                if( aceinfo.compressType==ZLib )
                        return fileinfomsg+"   2D texture ZLib compressed - skip";
                else
                        compmethod = ZLib2D;
         }
        else {
                if( !aceinfo.hasMIPMAP )
//                        ShowMessage("3D Texture not has MIP map!");
                        msglevel = 2;
         }


        // 3D Texture part

        // Message for ZLib compressed can not DXT recompress
        if( compmethod==NoCompress && aceinfo.compressType==ZLib &&
             ForceDXTCheckBox->Checked && !NoDXTCheckBox->Checked  ) {
                if( aceinfo.colorDepth==Alpha && !ForceTransCheckBox->Checked )
                        compmsg = "   No DXT for Alpha texture - skip";
                else if( !fnameallowdxt )
                        compmsg = "   No DXT format for this filename - skip";
                else if( aceinfo.width==1024 &&
                        !( UseMwddsCheckBox->Enabled && UseMwddsCheckBox->Checked ) &&
                        !UseAceItForAllCheckBox->Checked && !AceIt1024DXTCheckBox->Checked )
                        compmsg = "   1024 DXT format need AceIt - skip";
                else if( DXTZlibCheckBox->Checked )
                        compmethod = DXT1ZLib;
                else
                        compmethod = DXT1;
         }

        // Check does need ReMip ZLib texture
        if( compmethod==NoCompress && aceinfo.compressType==ZLib ) {
                if( ReMipItCheckBox->Checked ||
                    ( ReMipIt1024CheckBox->Checked && aceinfo.width==1024 ) ) {
                        if( !aceinfo.byAceIt() && !aceinfo.byReMipIt() )
                                compmethod = ReMip;
                        else
                                compmsg = "   Already ReMiped - skip";
                 }
         }


        // 3D Texture No compressed
        if( compmethod==NoCompress && aceinfo.compressType==NoCompress ) {
                if( NoDXTCheckBox->Checked || !fnameallowdxt )
                        compmethod = ZLib;
                else if( aceinfo.colorDepth==Alpha && !ForceTransCheckBox->Checked )
                        compmethod = ZLib;
                else if( aceinfo.width==1024 && !UseAceItForAllCheckBox->Checked &&
                                           !AceIt1024DXTCheckBox->Checked )
                        compmethod = ZLib;
                else if( DXTZlibCheckBox->Checked )
                        compmethod = DXT1ZLib;
                else
                        compmethod = DXT1;
         }


        // do compress using selected compress method
        if( compmethod!=NoCompress && !CheckOnlyCheckBox->Checked ) {
                msglevel = 1;
                compmsg = DoCompress(path,fname,filesize,aceinfo,
                                 compmethod,backupdir,msglevel);
                if( msglevel>1 ) Errors++;
         }
        if( compmethod!=NoCompress && CheckOnlyCheckBox->Checked ) {
                switch( compmethod ) {
                        case ZLib:
                                compmsg = "   ZLib compress";
                                break;
                        case ZLib2D:
                                compmsg = "   ZLib 2D compress";
                                break;
                        case DXT1:
                                if( aceinfo.compressType==ZLib )
                                        compmsg = "   DXT recompress";
                                else
                                        compmsg = "   DXT compress";
                                break;
                        case DXT1ZLib:
                                if( aceinfo.compressType==ZLib )
                                        compmsg = "   DXT & Zlib recompress";
                                else if( aceinfo.compressType==DXT1 )
                                        compmsg = "   DXT + Zlib compress";
                                else
                                        compmsg = "   DXT & Zlib compress";
                                break;
                        case ReMip:
                                compmsg = "   ReMip ZLib texture";
                                break;
                 }
                msglevel = 1;
         }
        return fileinfomsg+compmsg;
}
//---------------------------------------------------------------------------

// Decide use which program to compress
AnsiString TMainForm::DoCompress(AnsiString path,AnsiString fname,
                int filesize,AceInfo aceinfo,ACECompressType compmethod,
                AnsiString backupdir,int &msglevel)
{                            // compmethod = ZLib(3D),ZLib2D,DXT1,ReMip,DXT1ZLib
        AnsiString tganame,tganame2,destname,destname2,dxtname;
        AnsiString filepath,compmsg;
        bool needsquare,aceitselected,doremip;
        int filesize2;
        
        filepath = IncludeTrailingPathDelimiter(path)+fname;

        // Does use aceit.exe or remipit.exe
        aceitselected = UseAceItForAllCheckBox->Checked ||
                AceIt1024DXTCheckBox->Checked || AceIt1024ZLibCheckBox->Checked;
        // Does need ReMip whether ZLib compress or ReMip only
        doremip = compmethod==ReMip || ( compmethod==ZLib  &&
                  ( ReMipItCheckBox->Checked || ReMipIt1024CheckBox->Checked ));
        // Check does need square method convert to TGA file
        needsquare = !aceinfo.allow3D() && !UseAceItForAllCheckBox->Checked;


        // Create temporary TGA file name
        tganame = ChangeFileExt(filepath,".tga");
        if( FileExists(tganame) ) {
                tganame = ChangeFileExt(filepath,".tmp.tga");
                if( FileExists(tganame) )
                         DeleteFile(tganame);
          }
        tganame2 = ChangeFileExt(filepath,".temp.tga");
        if( FileExists(tganame2) )
                 DeleteFile(tganame2);
        // Create temporary destination ACE file name
        destname = ChangeFileExt(filepath,".ace.tmp");
        if( FileExists(destname) )
                 DeleteFile(destname);
        destname2 = ChangeFileExt(filepath,".ace.temp");
        if( FileExists(destname2) )
                 DeleteFile(destname2);

                 
        // Decompress
        if( !doremip || aceitselected || VerifyZLibCompCheckBox->Checked ) {
                if( !AceToTgaMwace(filepath,tganame,needsquare)
                        || !FileExists(tganame) ) {

                        Application->MessageBox(
                        ("Can not convert:\n"
                        +filepath+
                        "\to TGA file!").c_str(),
                        "Convert to TGA failed!",MB_OK+MB_ICONERROR);
                        msglevel = 2;
                        return "   Failed convert ACE file to TGA file!";
                 }
         }


        // Begin compress
        if( compmethod==ZLib2D ) {
                if( UseBuiltinZLibCompCheckBox->Checked )     // Use zlibwapi.dll ZLib first for speed
                        compmsg = CompDecompKujuFile(filepath,destname,true,
                                        StrToFloat(CompressRatioEdit->Text)*0.01,
                                        CompressLevelComboBox->ItemIndex+6);
                else if( UseAceItForAllCheckBox->Checked )
                        compmsg = TgaToAceUseAceIt(compmethod,tganame,
                                                destname,aceinfo.colorDepth);
                else {
                        if( aceinfo.colorDepth==Alpha &&
                                   ( aceinfo.width>512 || aceinfo.height>512 ) )
                                compmsg = " MakeACE.exe error handle Alpha texture when size>512  "
                                          "Please use builtin ZLib compress!";
                        else
                                compmsg = TgaToAceUseMakeAce(compmethod,tganame,destname,
                                        aceinfo.colorDepth,aceinfo.width,aceinfo.height);
                 }
         }

        if( compmethod==ZLib ) {
                if( UseAceItForAllCheckBox->Checked ||
                        ( aceinfo.width==1024 && AceIt1024ZLibCheckBox->Checked ) ||
                        ( aceitselected && doremip ) ) {
                        compmsg = TgaToAceUseAceIt(ZLib,tganame,destname,aceinfo.colorDepth);
                 }
                else if( doremip ) {
                        CopyFile(filepath.c_str(),destname.c_str(),false);
                        compmsg = ReMipAce(destname);           // ReMipIt do ZLib compress
                 }
                else if( UseBuiltinZLibCompCheckBox->Checked )     // Use zlibwapi.dll ZLib first than MakeACE for speed
                        compmsg = CompDecompKujuFile(filepath,destname,true,
                                        StrToFloat(CompressRatioEdit->Text)*0.01,
                                        CompressLevelComboBox->ItemIndex+6);
                else {
                        if( aceinfo.colorDepth==Alpha &&
                                   ( aceinfo.width>512 || aceinfo.height>512 ) )
                                compmsg = " MakeACE.exe error handle Alpha texture when size>512  "
                                          "Please use MakeACEWin.exe manually compress!";
                        else
                                compmsg = TgaToAceUseMakeAce(ZLib,tganame,destname,
                                             aceinfo.colorDepth,aceinfo.width,aceinfo.height);
                 }
         }


        if( compmethod==DXT1 || compmethod==DXT1ZLib ) {
                bool runaceit=false;
                if( aceinfo.compressType!=DXT1 ) {
                    if( UseMwddsCheckBox->Enabled && UseMwddsCheckBox->Checked ) {
                        compmsg = TgaToDXTAceUseMwdds(tganame,destname,
                                        aceinfo.colorDepth,DitherCheckBox->Checked,
                                        MipFilterComboBox->ItemIndex);
                     }
                    else if( UseAceItForAllCheckBox->Checked ||
                        ( aceinfo.width==1024 && AceIt1024DXTCheckBox->Checked )) {
                        runaceit=true;
                        compmsg = TgaToAceUseAceIt(compmethod,tganame,destname,aceinfo.colorDepth);
                     }
                    else {
                        compmsg = TgaToAceUseMakeAce(compmethod,tganame,destname,
                                aceinfo.colorDepth,aceinfo.width,aceinfo.height);
                     }

                    dxtname = destname;
                 }
                else
                    dxtname = filepath;


                // ZLib compress for DXT texture
                if( compmsg.IsEmpty() && compmethod==DXT1ZLib && FileExists(dxtname) ) {
                        filesize2 = getfilesize(dxtname);
                        if( aceinfo.compressType!=DXT1 &&
                            filesize2!=aceinfo.EstDXTFileSize() ) {
                                if( !runaceit ||  filesize2>aceinfo.EstDXTFileSize() )          // for new version AceIt auto ZLib for DXT1
                                        compmsg = "- DXT filesize incorrect - Abort!";
                         }
                        else {
                                compmsg = CompDecompKujuFile(dxtname,destname2,true,
                                        StrToFloat(CompressRatioEdit->Text)*0.01,
                                        CompressLevelComboBox->ItemIndex+6);

                                // delete temp DXT texture if ZLib compressed again
                                if( compmsg.IsEmpty() ) {
                                    destname = destname2;
                                    if( aceinfo.compressType!=DXT1 )
                                        if( !DeleteFile(dxtname) )
                                                compmsg = " Can not delete DXT temp file - Abort!";
                                 }
                         }
                 }
         }

        if( compmethod==ReMip ) {               // doremip always true
                if( aceitselected ) {
                        compmsg = TgaToAceUseAceIt(ZLib,tganame,destname,aceinfo.colorDepth);
                 }
                else {
                        CopyFile(filepath.c_str(),destname.c_str(),false);
                        compmsg = ReMipAce(destname);
                 }
         }

        // Check does there are error
        if( !compmsg.IsEmpty() ) {         // not null string mean error
                if( compmsg.SubString(1,4)!="SKIP" ) {
                        if( !KeepTempFileCheckBox->Checked ) {
                                DeleteFile(tganame);
                                DeleteFile(destname);
                                DeleteFile(destname2);
                         }
                        msglevel = 2;
                        return compmsg;
                 }
                else {          // ZLib compress canceled for ratio not enough
                        compmsg = compmsg.SubString(5,60);
                        if( compmethod==DXT1ZLib && aceinfo.compressType!=DXT1 )
                                compmsg = "  DXT1 compressed - "
                                        "ZLib compress ratio "+compmsg+" canceled!";
                        else {
                                DeleteFile(tganame);
                                DeleteFile(destname);
                                DeleteFile(destname2);
                                return "   ZLib compress canceled for ratio : "
                                        +compmsg+" not enough";
                         }
                 }
         }


        // Verify
        bool verifyok = true;
        AceInfo aceinfo2;

        filesize2 = getfilesize(destname);
        if( filesize2==0 ) {
                verifyok = false;
                compmsg += "- file size zero";
         }
        else if( filesize2==-1 ) {
                verifyok = false;
                compmsg += "- file not exist";
         }
        else if( compmethod==DXT1 ||
                 ( compmethod==DXT1ZLib && aceinfo.compressType!=DXT1 ) ) {
                AceFile acefile2;
                aceinfo2 = acefile2.GetInfo(destname);

                if( aceinfo.width!=aceinfo2.width || aceinfo.height!=aceinfo2.height
                        || ( aceinfo.colorDepth==Opaque && aceinfo2.colorDepth!=Opaque )
                        || ( aceinfo.colorDepth!=Opaque && aceinfo2.colorDepth==Opaque )
                        || aceinfo2.colorDepth==Unknown ) {
                        verifyok = false;
                        compmsg += "- DXT pic info not match";
                 }

                 if( compmethod==DXT1 && filesize2!=aceinfo.EstDXTFileSize() ) {
                        verifyok = false;
                        compmsg += "- DXT filesize incorrect: "+AnsiString(filesize2);
                  }
         }
        else if( VerifyZLibCompCheckBox->Checked ) {
                if( FileExists(tganame2) ) DeleteFile(tganame2);
                // Decompress compressed ACE for verify
                if( !AceToTgaMwace(destname,tganame2,needsquare) || !FileExists(tganame2) ) {
                        compmsg =  "- ZLib decompress failure";
                        verifyok = false;
                 }
                else {
                        if( comparefilebinary(tganame,tganame2)!=0 ) {
                                compmsg = "- ZLib pic content not match";
                                verifyok = false;
                                if( !KeepTempFileCheckBox->Checked )
                                        DeleteFile(tganame2);
                         }
                        else
                                DeleteFile(tganame2);
                  }
         }

        if( !verifyok ) {
                if( Application->MessageBox(
                                "Compressed File verify fail!\n"
                                "Old texture will not changed.\n"
                                "Do you want to abort process?",
                                "File verify fail!",MB_YESNO )==IDYES )
                        StopProcess = true;

                if( !KeepTempFileCheckBox->Checked ) {
                        DeleteFile(tganame);
                        DeleteFile(destname);
                 }
                msglevel = 2;
                return "    Compressed File verify fail! - not changed "+compmsg;
         }


        // Backup & rename
        if( !KeepTGAFileCheckBox->Checked ||
            !(DXTOnlyCheckBox->Checked && ( compmethod==DXT1 ||
                        ( compmethod==DXT1ZLib && aceinfo.compressType!=DXT1 ) ) ) )
                DeleteFile(tganame);
        else if( PutCompToBckDirCheckBox->Enabled && PutCompToBckDirCheckBox->Checked ) {
                tganame2 = IncludeTrailingPathDelimiter(backupdir)+fname;
                tganame2 = ChangeFileExt(tganame2,".tga");
                if( FileExists(tganame2) )
                        DeleteFile(tganame2);
                RenameFile(tganame,tganame2);
         }


        if( NoBackupRadioButton->Checked ||
            ( compmethod!=DXT1 && !( compmethod==DXT1ZLib && aceinfo.compressType!=DXT1 )
                        && NoBackupZLibCheckBox->Checked ) ) {
                if( !DeleteFile(filepath) ) {
                        StopProcess = true;

                        if( !KeepTempFileCheckBox->Checked ) DeleteFile(destname);
                        msglevel = 2;
                        return "    Delete old texture file fail! - process abort!";
                 }
         }
        else {
                if( AddExtBackupRadioButton->Checked )
                        destname2 = ChangeFileExt(filepath,".ace"+ExtEdit->Text);
                else if( MoveFileBackupRadioButton->Checked ) {
                        destname2 = IncludeTrailingPathDelimiter(backupdir)+fname;
                        ForceDirectories(backupdir);
                 }

                if( FileExists(destname2) ) {
                        if( Application->MessageBox(
                                ("Backup file: \n"+destname2+
                                "\n already exist!\n\n"
                                "Do you want to abort process?\n"
                                "Answer \"Yes\" to abort current process,\n"
                                "Answer \"No\" to overwrite backup file").c_str(),
                                "Backup file name exists!",MB_YESNO )==IDYES ) {

                                StopProcess = true;
                                if( !KeepTempFileCheckBox->Checked ) DeleteFile(destname);
                                msglevel = 2;
                                return "    Backup filename already exists! - user abort!";
                         }
                        else
                                DeleteFile(destname2);
                 }

                if( !PutCompToBckDirCheckBox->Enabled || !PutCompToBckDirCheckBox->Checked ) {
                        if( !RenameFile(filepath,destname2) ) {
                                StopProcess = true;
                                if( !KeepTempFileCheckBox->Checked ) DeleteFile(destname);
                                msglevel = 2;
                                return "    Backup texture file fail! - process abort!";
                         }
                 }
                else {                  // Special option for check only
                        if( !RenameFile(destname,destname2) ) {
                                StopProcess = true;
                                if( !KeepTempFileCheckBox->Checked ) DeleteFile(destname);
                                msglevel = 2;
                                return "    Put compressed texture to backup dir fail! - process abort!";
                         }
                 }
         }

        if( !PutCompToBckDirCheckBox->Enabled || !PutCompToBckDirCheckBox->Checked ) {
                if( !RenameFile(destname,filepath) ) {
                        StopProcess = true;
                        if( !KeepTempFileCheckBox->Checked ) DeleteFile(destname);
                        msglevel = 2;
                        return "    Rename new texture file fail! - process abort!";
                 }
         }


        // Message and statics
        if( compmsg.IsEmpty() ) {
                switch( compmethod ) {
                        case ZLib:
                                compmsg = "   ZLib compress";
                                break;
                        case ZLib2D:
                                compmsg = "   ZLib 2D compress";
                                break;
                        case DXT1:
                                if( aceinfo.compressType==ZLib )
                                        compmsg = "   DXT recompress";
                                else
                                        compmsg = "   DXT compress";

                                if( aceinfo.colorDepth==Alpha )
                                        compmsg += " - Force Trans!";

                                CompressDXT++;
                                break;
                        case DXT1ZLib:
                                if( aceinfo.compressType==ZLib )
                                        compmsg = "   DXT & ZLib recompress";
                                else if( aceinfo.compressType==DXT1 )
                                        compmsg = "   DXT + ZLib compress";
                                else
                                        compmsg = "   DXT & ZLib compress";

                                if( aceinfo.colorDepth==Alpha )
                                        compmsg += " - Force Trans!";

                                if( aceinfo.compressType!=DXT1 )
                                        CompressDXT++;
                                break;
                        case ReMip:
                                compmsg = "   ReMip texture";
                                break;
                 }
         }
        CompressFiles++;
        DiskSpaceSave += filesize-filesize2;
        compmsg += " - save "+AnsiString((filesize-filesize2+1023)/1024)+"kb";

        AfterFileTypeCount[aceinfo.compressType]--;
        AfterFileSizeSum[aceinfo.compressType] -= filesize;
        if( compmethod>DXT1ZLib )
                compmethod = ZLib;
        AfterFileTypeCount[compmethod]++;
        AfterFileSizeSum[compmethod] += filesize2;


        return compmsg;
}
//---------------------------------------------------------------------------

int TMainForm::CheckDirTextureType(AnsiString path)       // 0 2D, 1 3D, -1 Unknown
{
        SplitString(DirPath,ExcludeTrailingPathDelimiter(path),'\\');

        // Check does 2D texture
        if( CaseInsensCheck(DenyPDir,
                DirPath->Strings[DirPath->Count-1])!=-1 ) return is2D;
        if( DirPath->Count>=2 && CaseInsensCheck(DenyPPDir,
                DirPath->Strings[DirPath->Count-2])!=-1 ) return is2D;
        for(int i=0;i<DenyPathContainPhase->Count;i++)
                if( CaseInsensPos(path,DenyPathContainPhase->Strings[i]) )
                        return is2D;
        for(int i=0;i<DenyFileInSameDir->Count;i++)
                if( FileExists( IncludeTrailingPathDelimiter(path)
                                +DenyFileInSameDir->Strings[i] ) )
                        return is2D;

        // If not check then assume 3D texture
        if( !Check3DCheckBox->Checked )
                return is3D;

        // Check does 3D texture
        if( DirPath->Count>=2 ) {
                if( CaseInsensCheck(AllowPDir,
                        DirPath->Strings[DirPath->Count-1])!=-1
                    || CaseInsensCheck(AllowPPDir,
                        DirPath->Strings[DirPath->Count-2])!=-1 ) {

                        if( CaseInsensCheck(PathMustContain,
                                DirPath)!=-1 ) return is3D;
                         }
         }

        return typeUnknown;
}
//---------------------------------------------------------------------------
// This software is released under GNP General Public License (GPL)

